/*
 * ex14.cu
 * This is an exercise that aim to revise what you have learnt in previous exercises.
 */
#include <stdio.h>

void vector_add_cpu(const int *arr1, const int *arr2, int *results, int arrSize);
void verify_results(const int *resultsFromCPU, const int *resultsFromGPU, int arrSize);

__global__ void vector_add_kernel(const int *cudaArr1, const int *cudaArr2, int *cudaResults, int arrSize)
{
  int index = blockIdx.x * gridDim.y * blockDim.x * blockDim.y + blockIdx.y * blockDim.x * blockDim.y + threadIdx.x * blockDim.x + threadIdx.y;
    
  cudaResults[index] = cudaArr1[index] + cudaArr2[index];
}

int main(int argc, char *argv[])
{
  int i;
  int arrSize = 64;

  int *arr1, *arr2;
  int *resultsFromCPU;

  int *cudaArr1, *cudaArr2;
  int *cudaResults;
  int *resultsFromGPU;

  /* Allocate CPU memory */
  arr1 = (int *)malloc( arrSize * sizeof(int) );
  arr2 = (int *)malloc( arrSize * sizeof(int) );
  resultsFromCPU = (int *)malloc( arrSize * sizeof(int) );
  resultsFromGPU = (int *)malloc( arrSize * sizeof(int) );

  /* Initialize vectors */
  for( i = 0; i < arrSize; i++ )
  {
    arr1[i] = (int) (i * 2);
    arr2[i] = (int) ((i * 2) + 1);
  }

  vector_add_cpu(arr1, arr2, resultsFromCPU, arrSize);

  /********************************************************/
  /* Write your implementation below */
  /********************************************************/
  cudaMalloc( (void **)&cudaArr1, arrSize * sizeof(int) );
  cudaMalloc( (void **)&cudaArr2, arrSize * sizeof(int) );
  cudaMalloc( (void **)&cudaResults, arrSize * sizeof(int) );

  cudaMemset( cudaResults, 0, arrSize * sizeof(int) );

  cudaMemcpy( cudaArr1, arr1, arrSize * sizeof(int), cudaMemcpyHostToDevice );
  cudaMemcpy( cudaArr2, arr2, arrSize * sizeof(int), cudaMemcpyHostToDevice );

  dim3 threads(2, 2);
  dim3 blocks( 2, 8 );

  vector_add_kernel<<< blocks, threads >>>(cudaArr1, cudaArr2, cudaResults, arrSize);

  cudaMemcpy( resultsFromGPU, cudaResults, arrSize * sizeof(int), cudaMemcpyDeviceToHost );


  verify_results(resultsFromCPU, resultsFromGPU, arrSize);

  /* Free your GPU memory below */
  cudaFree(cudaArr1);
  cudaFree(cudaArr2);
  cudaFree(cudaResults);
  
  cudaDeviceReset();

  /* Free CPU memory */
  free(arr1);
  free(arr2);
  free(resultsFromCPU);

  return 0;
}

/* CPU implementation of vector addition */
void vector_add_cpu(const int *arr1, const int *arr2, int *resultsFromCPU, int arrSize)
{
  int i;
  for( i = 0; i < arrSize; i++ )
    resultsFromCPU[i] = arr1[i] + arr2[i];
}

void verify_results(const int *resultsFromCPU, const int *resultsFromGPU, int arrSize)
{
  int i;

  for( i = 0; i < arrSize; i++ )
  {
    //printf( "%d - CPU: %d, GPU: %d\n", i, resultsFromCPU[i], resultsFromGPU[i] );
    if( resultsFromCPU[i] != resultsFromGPU[i] )
    {
      printf( "Sorry, wrong answers, try again.\n" );
      exit(EXIT_FAILURE);
    }
  }
  
  printf( "Congratulations, you got it!\n" );
}
