/*
  ex35.cu
  This is an exercise to demonstrate the usage of cudaMallocPitch().
*/
#include <stdio.h>

__global__ void simple_kernel(float *input, float *output, size_t pitch)
{
  int i = blockIdx.x * pitch / sizeof(float) + threadIdx.x;

  output[i] = input[i];
}

int main(int argc, char *argv[])
{
  int i, j;
  /* Arrays in host memory */
  float **arr2D;
  float *arr1D;
  float *results;

  /* Arrays in device memory */
  float *cudaArr;
  float *cudaResults;

  int nrows = 8;
  int ncols = 4;

  size_t pitchArr;
  size_t pitchResults;

  if( argc > 1 ) ncols = atoi(argv[1]);

  /* Allocate 2D array in CPU */
  arr2D = (float **)malloc( nrows * sizeof(float *) );
  for( i = 0; i < nrows; i++ )
    arr2D[i] = (float *)malloc( ncols * sizeof(float) );

  for( i = 0; i < nrows; i++ )
  {
    for( j = 0 ; j < ncols; j++ )
    {
      arr2D[i][j] = (float) (i + j); 
    }
  }

  /* Allocate a single pointer and copy in the contents */
  arr1D = (float *)malloc( nrows * ncols * sizeof(float) );
  results = (float *)malloc( nrows * ncols * sizeof(float) );

  for( i = 0; i < nrows; i++ )
    memcpy( &arr1D[i * ncols], arr2D[i], ncols * sizeof(float) );

  /* Allocate GPU memory */
  cudaMallocPitch( (void **)&cudaArr, &pitchArr, 
      ncols * sizeof(float), nrows );
  cudaMallocPitch( (void **)&cudaResults, &pitchResults,
      ncols * sizeof(float), nrows );

  /* Transfer the data to the GPU */
  cudaMemcpy2D( cudaArr, pitchArr, arr1D, ncols * sizeof(float),
      ncols * sizeof(float), nrows,
      cudaMemcpyHostToDevice );
  
  simple_kernel<<< nrows, ncols >>>(cudaArr, cudaResults, pitchArr);

  /* Memory transfer out */
  cudaMemcpy2D( results, ncols * sizeof(float), cudaResults, pitchResults,
      ncols * sizeof(float), nrows,
      cudaMemcpyDeviceToHost );

  /* Print the results */
  printf( "Result matrix\n" );
  for( i = 0; i < nrows; i++ )
  {
    for( j = 0; j < ncols; j++ )
      printf( "%f ", results[i * ncols + j] );
    printf( "\n" );
  }
  printf( "Pitch: %d\n", pitchArr );

  /* Free the CPU 2D array */
  for( i = 0; i < nrows; i++ )
    free(arr2D[i]);
  free(arr2D);

  free(results);
  free(arr1D);
  cudaFree(cudaArr);
  cudaFree(cudaResults);

  return 0;
}
