/*
   ex15.cu
   This is an example of how to time your functions.
 */
#include <sys/time.h>
#include <stdio.h>

int main(int argc, char *argv[])
{
  int i;
  float a;

  /* Variables for timing function using CPU time stamp (for Linux) */
  struct timeval tvstart, tvstop;

  /* Variables for timing function using the GPU clock time */
  cudaEvent_t start, stop;
  float time;
  cudaEventCreate(&start);
  cudaEventCreate(&stop);

  /* Time a single for loop with CPU time stamp */
  gettimeofday(&tvstart, NULL);

  for( i = 0; i < 1000000; i++ )
  {
    a = a + 0.1f;
  }

  /* For timing CUDA functions, uncomment the line below */
  // cudaDeviceSynchronize();

  gettimeofday(&tvstop, NULL);
  printf( "CPU time measurement: %f\n", 1.e3f*(float)(tvstop.tv_sec-tvstart.tv_sec) + 1.e-3f*(float)(tvstop.tv_usec-tvstart.tv_usec) );

  /* Timing using GPU clock time */
  cudaEventRecord(start, 0);

  for( i = 0; i < 1000000; i++ )
  {
    a = a + 0.2f;
  }

  cudaEventRecord(stop, 0);
  cudaEventSynchronize(stop);

  cudaEventElapsedTime( &time, start, stop );
  printf( "GPU time measurement: %f\n", time );

  cudaEventDestroy(start);
  cudaEventDestroy(stop);

  return 0;
}
