/*
 * ex21.cu
 * This is the exercise for learning the usage of CUBLAS library.
 */

#include <stdio.h>
#include <stdlib.h>
#include <cublas_v2.h>

int main(int argc, char *argv[])
{
  int i;
  /* Vector used for scaling examples */
  float *arr;
  float *cudaArr;
  int arrSize = 32;
  float scalingFactor = 0.5;

  /* Vectors used for dot product exercises */
  double *vec1;
  double *vec2;
  double *cudaVec1;
  double *cudaVec2;
  double result;
  int vecSize = 16;

  /* Allocate memory */
  arr = (float *)malloc( arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaArr, arrSize * sizeof(float) );

  vec1 = (double *)malloc( vecSize * sizeof(double) );
  vec2 = (double *)malloc( vecSize * sizeof(double) );
  cudaMalloc( (void **)&cudaVec1, vecSize * sizeof(double) );
  cudaMalloc( (void **)&cudaVec2, vecSize * sizeof(double) );

  /* Initialize the values of arr */
  for( i = 0; i < arrSize; i++ )
  {
    arr[i] = (float) (i * 16);
  }

  /* Initialize the values of vec1 and vec2 */
  for( i = 0; i < vecSize; i++ )
  {
    vec1[i] = (double) ((i + 10) * 2);
    vec2[i] = (double) ((i+4) * 2);
  }

  /* Start calling CUBLAS functions */
  cublasHandle_t handle;

  cublasCreate(&handle);

  /* Scale the array by a scaling factor */
  /* First print out the arr items before scaling */
  for( i = 0; i < arrSize; i++ )
    printf( "%f\n", arr[i] );
  printf( "\n" );
  cublasSetVector(arrSize, sizeof(float), arr, 1, cudaArr, 1);

  cublasSscal( handle, arrSize, &scalingFactor, cudaArr, 1 );

  cublasGetVector(arrSize, sizeof(float), cudaArr, 1, arr, 1);

  /* Now print out the results after scaling */
  printf( "After Scaling\n" );
  for( i = 0; i < arrSize; i++ )
    printf( "%f\n", arr[i] );

  /*****************************************************/
  /* Insert your code that uses CUBLAS dot product function */
  /*****************************************************/




  /* End of dot product */

  cublasDestroy(handle);

  free(arr);
  free(vec1);
  free(vec2);

  cudaFree(cudaArr);
  cudaFree(cudaVec1);
  cudaFree(cudaVec2);

  return 0;
}
