/*
  ex22.cu
  This is an exercise for learning about usage of CUFFT.
  The routine for FFTW can be found in exfftw.c, it was 
  separated from this file because the FFTW functions cannot be
  linked from a .cu file. 
*/
#include <stdio.h>
#include <complex.h>
#include <cufft.h>

/* C wrapper function for FFTW, setup and destroy variable are used to determine if you need to setup or destroy the plan in the FFTW function */
extern "C" void runFFTW(float *input, float complex *out, int size, int setup, int destroy);

int main(int argc, char *argv[])
{
  int i;
  float *input;
  
  /* Variables in host memory */
  complex float *outputFromFFTW;
  complex float *outputFromCUDA;

  /* Variables in device memory */
  float *cudaInput;
  float complex *cudaOutput;

  int size = 8;

  /* Allocate host memory */
  input = (float *)malloc( size * sizeof(float) );
  outputFromFFTW = (float complex *)malloc( (size/2 + 1) * sizeof(float complex) );
  outputFromCUDA = (float complex *)malloc( (size/2 + 1) * sizeof(float complex) );

  /* Initialize input */
  for( i = 0; i < size; i++ )
    input[i] = (float) (i+1);

  runFFTW(input, outputFromFFTW, size, 1, 1);

  /* Print out the results of FFT, comment out the printf if 
   you use very large data size */
  for( i = 0; i < size/2 + 1; i++ )
    printf( "%f %f\n", crealf(outputFromFFTW[i]), cimagf(outputFromFFTW[i]) );
	

  /*******************************************************/
  /* Insert you CUFFT implementation below, 
     using "input" as input and "outputFromCUDA" as output.
     You can write in a similar function as the FFTW,
     or just put down the code below.
   */
  /*******************************************************/


  return 0;
}


