/*
  ex34.cu. This is an example to demonstrate the effect of misaligned access.
  This software contains source code provided by NVIDIA Corporation.
*/
#include <stdio.h>

#define OFFSETLIMIT 1024
__global__ void offsetCopy(float *odata, float *idata, int offset)
{
  int xid = blockIdx.x * blockDim.x + threadIdx.x + offset;

  odata[xid] = idata[xid];
}

int main(int argc, char *argv[])
{
  int i;
  int arrSize = 16 * 1024 * 1024 + OFFSETLIMIT;
  int offset = 0;

  float *input;

  float *cudaInput;
  float *cudaOutput;

  if( argc > 1 ) offset = atoi(argv[1]);

  input = (float *)malloc( arrSize * sizeof(float) );

  cudaMalloc( (void **)&cudaInput, arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaOutput, arrSize * sizeof(float) );

  /* Initialize array */
  for( i = 0; i < arrSize; i++ )
    input[i] = (float) (i * 0.1);

  cudaMemcpy( cudaInput, input, arrSize * sizeof(float), 
      cudaMemcpyHostToDevice );

  dim3 threads( 512, 1 );
  dim3 blocks( (arrSize - OFFSETLIMIT) / threads.x, 1 );

  offsetCopy<<< blocks, threads >>>(cudaOutput, cudaInput, offset);

  cudaFree(cudaInput);
  cudaFree(cudaOutput);

  return 0;
}

