/*
   ex36.cu
   This is an example to demonstrate the usage of shared memory.
 */

#include <stdio.h>

__global__ void sharedmem_kernel(float *input, float *output)
{
  int i;
  float temp = 0.0f;
  //__shared__ float shared[1];
  /* Can also be declare like above, which uses a known number (or macro) */
  extern __shared__ float shared[];

  /* Read the global memory into shared memory, one thread will load only one memory */ 
  if( threadIdx.x < gridDim.x )
    shared[threadIdx.x] = input[threadIdx.x];

  int index = blockIdx.x * blockDim.x + threadIdx.x;
  __syncthreads();

  /* Now do some operations onto the shared memory and save back to results */
  for( i = 0; i < gridDim.x; i++ )
    temp += shared[i] * (threadIdx.x + blockIdx.x);
  output[index] = temp;
}

__global__ void globalmem_kernel(float *input, float *output)
{
  int i;
  float temp = 0.0f;
  int index = blockIdx.x * blockDim.x + threadIdx.x;

  for( i = 0; i < gridDim.x; i++ )
    temp += input[i] * (threadIdx.x + blockIdx.x);

  output[index] = temp;
}

int main(int argc, char *argv[])
{
  int i;
  int arrSize = 128;

  /* Randomly picked kernel configuration */
  int nthreads = arrSize * 4;
  int nblocks = arrSize;

  /* Arrays in CPU memory */
  float *arr;

  /* Arrays in GPU memory */
  float *cudaArr;
  float *cudaResults1;
  float *cudaResults2;

  arr = (float *)malloc( arrSize * sizeof(float) );

  cudaMalloc( (void **)&cudaArr, arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaResults1, nthreads * nblocks * sizeof(float) );
  cudaMalloc( (void **)&cudaResults2, nthreads * nblocks * sizeof(float) );

  for( i = 0; i < arrSize; i++ )
    arr[i] = (float) (i * 0.5);

  cudaMemcpy( cudaArr, arr, arrSize * sizeof(float),
      cudaMemcpyHostToDevice );

  /* Note the extra configuration variable to allocating shared memory space */
  sharedmem_kernel<<< nblocks, nthreads, arrSize * sizeof(float) >>>(cudaArr, cudaResults2);
  
  globalmem_kernel<<< nblocks, nthreads >>>(cudaArr, cudaResults1);
  
  cudaDeviceReset();

  free(arr);

  cudaFree(cudaArr);
  cudaFree(cudaResults1);
  cudaFree(cudaResults2);
  return 0;
}
