/*
   ex36_ma.cu
   This is an exercise to practice the usage of shared memory, by calculating a simple moving average.
 */

#include <stdio.h>
#include <math.h>

__global__ void mashared_kernel(float *input, float *output, int ndays)
{
}

__global__ void maglobal_kernel(float *input, float *output, int ndays)
{
  int i;
  float temp = 0.0f;
  int index = blockIdx.x * blockDim.x + threadIdx.x;

  for( i = index; i < index+ndays; i++ )
    temp += input[i];

  output[index] = temp / ndays;
}

int main(int argc, char *argv[])
{
  int i;

  /* This is the number of data will be used for each moving average */
  int ndays = 8;

  if( argc > 1 ) ndays = atoi(argv[1]);

  /* Say we have this many days of financial data */
  int arrSize = 1024*1024 + ndays;
  /* Need to calculate this many ndays */
  int numMA = arrSize - ndays;

  int nthreads = 512;
  int nblocks = numMA / nthreads;

  /* Used for calculating max difference between two arrays */
  float max_diff;

  /* Arrays in CPU memory */
  float *arr;
  float *results1;
  float *results2;

  /* Arrays in GPU memory */
  float *cudaArr;
  float *cudaResults1;
  float *cudaResults2;

  arr = (float *)malloc( arrSize * sizeof(float) );
  results = (float *)malloc( numMA * sizeof(float) );

  cudaMalloc( (void **)&cudaArr, arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaResults1, numMA * sizeof(float) );
  cudaMalloc( (void **)&cudaResults2, numMA * sizeof(float) );

  for( i = 0; i < arrSize; i++ )
    arr[i] = (float) (i * 1.2);

  cudaMemcpy( cudaArr, arr, arrSize * sizeof(float),
      cudaMemcpyHostToDevice );

  maglobal_kernel<<< nblocks, nthreads >>>(cudaArr, cudaResults1, ndays);
  
  cudaMemcpy( results1, cudaResults1, numMA * sizeof(float), 
      cudaMemcpyDeviceToHost );

  mashared_kernel<<< nblocks, nthreads, (nthreads+ndays)*sizeof(float) >>>( cudaArr, cudaResults2, ndays );

  cudaMemcpy( results2, cudaResults2, numMA * sizeof(float), 
      cudaMemcpyDeviceToHost );

  max_diff = calculate_max_difference(results1, results2, numMA);

  printf( "Maximum difference between the two method is: %e\n", max_diff );
  cudaDeviceReset();

  free(arr);
  free(results1);
  free(results2);

  cudaFree(cudaArr);
  cudaFree(cudaResults1);
  cudaFree(cudaResults2);

  return 0;
}

float calculate_max_difference(float *arr1, float *arr2, int size)
{
  int i;

  float max;

  for( i = 0; i < size; i++ )
  {
    if( fabs(arr1[i] - arr2[i]) > max )
      max = fabs(arr1[i]-arr2[i])
  }
  return max;
}

