/*
   ex42.cu
   This example demonstrate how the bank conflict occurs when reading from shared memory.
*/
#include <stdio.h>

/* To avoid the need to load some extra libraries and use extra functions, we will define our own struct */

/* Kernel for accessing the complex number */
__global__ void simple_kernel(float *inputre, float *inputim, float *outputre, float *outputim, int niterations)
{
  int j;
  int i = blockIdx.x * blockDim.x + threadIdx.x;

  for( j = 0; j < niterations; j++ )
  {
    outputre[i] += inputre[i];
    outputim[i] += inputim[i];
  }
}

int main(int argc, char *argv[])
{
  int i;

  int niterations = 10;
  
  int nthreads = 256;
  int nblocks = 2048;

  int arrSize = nthreads * nblocks;

  float *inputre;
  float *inputim;

  float *cudaInputre;
  float *cudaInputim;
  float *cudaOutputre;
  float *cudaOutputim;

  inputre = (float *)malloc( arrSize * sizeof(float) );
  inputim = (float *)malloc( arrSize * sizeof(float) );

  cudaMalloc( (void **)&cudaInputre, arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaInputim, arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaOutputre, arrSize * sizeof(float) );
  cudaMalloc( (void **)&cudaOutputim, arrSize * sizeof(float) );

  /* Initialize input array */
  for( i = 0; i < arrSize; i++ )
  {
    inputre[i] = (float) (i + 1);
    inputim[i] = (float) (i - 10);
  }

  cudaMemcpy( cudaInputre, inputre, arrSize * sizeof(float), 
      cudaMemcpyHostToDevice );
  cudaMemcpy( cudaInputim, inputim, arrSize * sizeof(float), 
      cudaMemcpyHostToDevice );

  simple_kernel<<< nblocks, nthreads >>>(cudaInputre, cudaInputim, 
      cudaOutputre, cudaOutputim, niterations);
  free(inputre);
  free(inputim);
  cudaFree(cudaInputre);
  cudaFree(cudaInputim);
  cudaFree(cudaOutputre);
  cudaFree(cudaOutputim);

  return 0;
}
